@@ -2,18 +2,17 @@
 
 from __future__ import annotations
 
-from math import ceil
-from time import monotonic
-from typing import Callable, Optional
+from typing import Optional
 
 from rich.style import Style
 
 from .._types import UnusedParameter
 from ..app import ComposeResult, RenderResult
+from ..clock import Clock
+from ..eta import ETA
 from ..geometry import clamp
 from ..reactive import reactive
 from ..renderables.bar import Bar as BarRenderable
-from ..timer import Timer
 from ..widget import Widget
 from ..widgets import Label
 
@@ -58,10 +57,8 @@ class Bar(Widget, can_focus=False):
     }
     """
 
-    _percentage: reactive[float | None] = reactive[Optional[float]](None)
+    percentage: reactive[float | None] = reactive[Optional[float]](None)
     """The percentage of progress that has been completed."""
-    _start_time: float | None
-    """The time when the widget started tracking progress."""
 
     def __init__(
         self,
@@ -69,13 +66,22 @@ class Bar(Widget, can_focus=False):
         id: str | None = None,
         classes: str | None = None,
         disabled: bool = False,
+        clock: Clock | None = None,
     ):
         """Create a bar for a [`ProgressBar`][textual.widgets.ProgressBar]."""
+        self._clock = (clock or Clock()).clone()
         super().__init__(name=name, id=id, classes=classes, disabled=disabled)
-        self._start_time = None
-        self._percentage = None
 
-    def watch__percentage(self, percentage: float | None) -> None:
+    def _validate_percentage(self, percentage: float | None) -> float | None:
+        """Avoid updating the bar, if the percentage increase is too small to render."""
+        width = self.size.width * 2
+        return (
+            None
+            if percentage is None
+            else (int(percentage * width) / width if width else percentage)
+        )
+
+    def watch_percentage(self, percentage: float | None) -> None:
         """Manage the timer that enables the indeterminate bar animation."""
         if percentage is not None:
             self.auto_refresh = None
@@ -84,16 +90,16 @@ class Bar(Widget, can_focus=False):
 
     def render(self) -> RenderResult:
         """Render the bar with the correct portion filled."""
-        if self._percentage is None:
+        if self.percentage is None:
             return self.render_indeterminate()
         else:
             bar_style = (
                 self.get_component_rich_style("bar--bar")
-                if self._percentage < 1
+                if self.percentage < 1
                 else self.get_component_rich_style("bar--complete")
             )
             return BarRenderable(
-                highlight_range=(0, self.size.width * self._percentage),
+                highlight_range=(0, self.size.width * self.percentage),
                 highlight_style=Style.from_color(bar_style.color),
                 background_style=Style.from_color(bar_style.bgcolor),
             )
@@ -104,14 +110,15 @@ class Bar(Widget, can_focus=False):
         highlighted_bar_width = 0.25 * width
         # Width used to enable the visual effect of the bar going into the corners.
         total_imaginary_width = width + highlighted_bar_width
-
+        start: float
+        end: float
         if self.app.animation_level == "none":
             start = 0
             end = width
         else:
             speed = 30  # Cells per second.
             # Compute the position of the bar.
-            start = (speed * self._get_elapsed_time()) % (2 * total_imaginary_width)
+            start = (speed * self._clock.time) % (2 * total_imaginary_width)
             if start > total_imaginary_width:
                 # If the bar is to the right of its width, wrap it back from right to left.
                 start = 2 * total_imaginary_width - start  # = (tiw - (start - tiw))
@@ -125,21 +132,6 @@ class Bar(Widget, can_focus=False):
             background_style=Style.from_color(bar_style.bgcolor),
         )
 
-    def _get_elapsed_time(self) -> float:
-        """Get time for the indeterminate progress animation.
-
-        This method ensures that the progress bar animation always starts at the
-        beginning and it also makes it easier to test the bar if we monkey patch
-        this method.
-
-        Returns:
-            The time elapsed since the bar started being animated.
-        """
-        if self._start_time is None:
-            self._start_time = monotonic()
-            return 0
-        return monotonic() - self._start_time
-
 
 class PercentageStatus(Label):
     """A label to display the percentage status of the progress bar."""
@@ -151,32 +143,14 @@ class PercentageStatus(Label):
     }
     """
 
-    _label_text: reactive[str] = reactive("", repaint=False)
-    """This is used as an auxiliary reactive to only refresh the label when needed."""
-    _percentage: reactive[float | None] = reactive[Optional[float]](None)
+    percentage: reactive[int | None] = reactive[Optional[int]](None)
     """The percentage of progress that has been completed."""
 
-    def __init__(
-        self,
-        name: str | None = None,
-        id: str | None = None,
-        classes: str | None = None,
-        disabled: bool = False,
-    ):
-        super().__init__(name=name, id=id, classes=classes, disabled=disabled)
-        self._percentage = None
-        self._label_text = "--%"
+    def _validate_percentage(self, percentage: float | None) -> int | None:
+        return None if percentage is None else round(percentage * 100)
 
-    def watch__percentage(self, percentage: float | None) -> None:
-        """Manage the text that shows the percentage of progress."""
-        if percentage is None:
-            self._label_text = "--%"
-        else:
-            self._label_text = f"{int(100 * percentage)}%"
-
-    def watch__label_text(self, label_text: str) -> None:
-        """If the label text changed, update the renderable (which also refreshes)."""
-        self.update(label_text)
+    def render(self) -> RenderResult:
+        return "--%" if self.percentage is None else f"{self.percentage}%"
 
 
 class ETAStatus(Label):
@@ -188,79 +162,23 @@ class ETAStatus(Label):
         content-align-horizontal: right;
     }
     """
+    eta: reactive[float | None] = reactive[Optional[float]](None)
+    """Estimated number of seconds till completion, or `None` if no estimate is available."""
 
-    _label_text: reactive[str] = reactive("", repaint=False)
-    """This is used as an auxiliary reactive to only refresh the label when needed."""
-    _percentage: reactive[float | None] = reactive[Optional[float]](None)
-    """The percentage of progress that has been completed."""
-    _refresh_timer: Timer | None
-    """Timer to update ETA status even when progress stalls."""
-    _start_time: float | None
-    """The time when the widget started tracking progress."""
-
-    def __init__(
-        self,
-        name: str | None = None,
-        id: str | None = None,
-        classes: str | None = None,
-        disabled: bool = False,
-    ):
-        super().__init__(name=name, id=id, classes=classes, disabled=disabled)
-        self._percentage = None
-        self._label_text = "--:--:--"
-        self._start_time = None
-        self._refresh_timer = None
-
-    def on_mount(self) -> None:
-        """Periodically refresh the countdown so that the ETA is always up to date."""
-        self._refresh_timer = self.set_interval(1 / 2, self.update_eta, pause=True)
-
-    def watch__percentage(self, percentage: float | None) -> None:
-        if percentage is None:
-            self._label_text = "--:--:--"
+    def render(self) -> RenderResult:
+        """Render the ETA display."""
+        eta = self.eta
+        if eta is None:
+            return "--:--:--"
         else:
-            if self._refresh_timer is not None:
-                self._refresh_timer.reset()
-            self.update_eta()
-
-    def update_eta(self) -> None:
-        """Update the ETA display."""
-        percentage = self._percentage
-        delta = self._get_elapsed_time()
-        # We display --:--:-- if we haven't started, if we are done,
-        # or if we don't know when we started keeping track of time.
-        if not percentage or percentage >= 1 or not delta:
-            self._label_text = "--:--:--"
-            # If we are done, we can delete the timer that periodically refreshes
-            # the countdown display.
-            if percentage is not None and percentage >= 1:
-                self.auto_refresh = None
-        # Render a countdown timer with hh:mm:ss, unless it's a LONG time.
-        else:
-            left = ceil((delta / percentage) * (1 - percentage))
-            minutes, seconds = divmod(left, 60)
+            minutes, seconds = divmod(round(eta), 60)
             hours, minutes = divmod(minutes, 60)
             if hours > 999999:
-                self._label_text = "+999999h"
+                return "+999999h"
             elif hours > 99:
-                self._label_text = f"{hours}h"
+                return f"{hours}h"
             else:
-                self._label_text = f"{hours:02}:{minutes:02}:{seconds:02}"
-
-    def _get_elapsed_time(self) -> float:
-        """Get time to estimate time to progress completion.
-
-        Returns:
-            The time elapsed since the bar started being animated.
-        """
-        if self._start_time is None:
-            self._start_time = monotonic()
-            return 0
-        return monotonic() - self._start_time
-
-    def watch__label_text(self, label_text: str) -> None:
-        """If the ETA label changed, update the renderable (which also refreshes)."""
-        self.update(label_text)
+                return f"{hours:02}:{minutes:02}:{seconds:02}"
 
 
 class ProgressBar(Widget, can_focus=False):
@@ -296,6 +214,7 @@ class ProgressBar(Widget, can_focus=False):
         print(progress_bar.percentage)  # 0.5
         ```
     """
+    _display_eta: reactive[int | None] = reactive[Optional[int]](None)
 
     def __init__(
         self,
@@ -308,6 +227,7 @@ class ProgressBar(Widget, can_focus=False):
         id: str | None = None,
         classes: str | None = None,
         disabled: bool = False,
+        clock: Clock | None = None,
     ):
         """Create a Progress Bar widget.
 
@@ -332,47 +252,28 @@ class ProgressBar(Widget, can_focus=False):
             id: The ID of the widget in the DOM.
             classes: The CSS classes for the widget.
             disabled: Whether the widget is disabled or not.
+            clock: An optional clock object (leave as default unless testing).
         """
+        self._clock = clock or Clock()
+        self._eta = ETA()
         super().__init__(name=name, id=id, classes=classes, disabled=disabled)
+        self.total = total
         self.show_bar = show_bar
         self.show_percentage = show_percentage
         self.show_eta = show_eta
 
-        self.total = total
+    def on_mount(self) -> None:
+        self.update()
+        self.set_interval(1, self.update)
+        self._clock.reset()
 
     def compose(self) -> ComposeResult:
-        # We create a closure so that we can determine what are the sub-widgets
-        # that are present and, therefore, will need to be notified about changes
-        # to the percentage.
-        def update_percentage(
-            widget: Bar | PercentageStatus | ETAStatus,
-        ) -> Callable[[float | None], None]:
-            """Closure to allow updating the percentage of a given widget."""
-
-            def updater(percentage: float | None) -> None:
-                """Update the percentage reactive of the enclosed widget."""
-                widget._percentage = percentage
-
-            return updater
-
         if self.show_bar:
-            bar = Bar(id="bar")
-            self.watch(self, "percentage", update_percentage(bar))
-            yield bar
+            yield Bar(id="bar", clock=self._clock).data_bind(ProgressBar.percentage)
         if self.show_percentage:
-            percentage_status = PercentageStatus(id="percentage")
-            self.watch(self, "percentage", update_percentage(percentage_status))
-            yield percentage_status
+            yield PercentageStatus(id="percentage").data_bind(ProgressBar.percentage)
         if self.show_eta:
-            eta_status = ETAStatus(id="eta")
-            self.watch(self, "percentage", update_percentage(eta_status))
-            yield eta_status
-
-    def _validate_progress(self, progress: float) -> float:
-        """Clamp the progress between 0 and the maximum total."""
-        if self.total is not None:
-            return clamp(progress, 0, self.total)
-        return progress
+            yield ETAStatus(id="eta").data_bind(eta=ProgressBar._display_eta)
 
     def _validate_total(self, total: float | None) -> float | None:
         """Ensure the total is not negative."""
@@ -380,21 +281,21 @@ class ProgressBar(Widget, can_focus=False):
             return total
         return max(0, total)
 
-    def _watch_total(self) -> None:
-        """Re-validate progress."""
-        self.progress = self.progress
-
     def _compute_percentage(self) -> float | None:
         """Keep the percentage of progress updated automatically.
 
         This will report a percentage of `1` if the total is zero.
         """
         if self.total:
-            return self.progress / self.total
+            return clamp(self.progress / self.total, 0.0, 1.0)
         elif self.total == 0:
-            return 1
+            return 1.0
         return None
 
+    def _watch_progress(self, progress: float) -> None:
+        """Perform update when progress is modified."""
+        self.update(progress=progress)
+
     def advance(self, advance: float = 1) -> None:
         """Advance the progress of the progress bar by the given amount.
 
@@ -406,7 +307,7 @@ class ProgressBar(Widget, can_focus=False):
         Args:
             advance: Number of steps to advance progress by.
         """
-        self.progress += advance
+        self.update(advance=advance)
 
     def update(
         self,
@@ -430,9 +331,25 @@ class ProgressBar(Widget, can_focus=False):
             progress: Set the progress to the given number of steps.
             advance: Advance the progress by this number of steps.
         """
+        current_time = self._clock.time
         if not isinstance(total, UnusedParameter):
+            if total is None or total != self.total:
+                self._eta.reset()
             self.total = total
+
+        def add_sample() -> None:
+            """Add a new sample."""
+            if self.progress is not None and self.total:
+                self._eta.add_sample(current_time, self.progress / self.total)
+
         if not isinstance(progress, UnusedParameter):
             self.progress = progress
+            add_sample()
+
         if not isinstance(advance, UnusedParameter):
             self.progress += advance
+            add_sample()
+
+        self._display_eta = (
+            None if self.total is None else self._eta.get_eta(current_time)
+        )
